from nose.tools import assert_true, assert_false, assert_equals, with_setup
from gixy.core.context import get_context, push_context, purge_context
from gixy.directives.block import Root
from gixy.core.regexp import Regexp
from gixy.core.variable import Variable

def setup():
    push_context(Root())


def tear_down():
    purge_context()


@with_setup(setup, tear_down)
def test_literal():
    var = Variable(name='simple', value='$uri', have_script=False)
    assert_false(var.depends)
    assert_false(var.regexp)
    assert_equals(var.value, '$uri')

    assert_false(var.can_startswith('$'))
    assert_false(var.can_contain('i'))
    assert_true(var.must_contain('$'))
    assert_true(var.must_contain('u'))
    assert_false(var.must_contain('a'))
    assert_true(var.must_startswith('$'))
    assert_false(var.must_startswith('u'))


@with_setup(setup, tear_down)
def test_regexp():
    var = Variable(name='simple', value=Regexp('^/.*'))
    assert_false(var.depends)
    assert_true(var.regexp)

    assert_true(var.can_startswith('/'))
    assert_false(var.can_startswith('a'))
    assert_true(var.can_contain('a'))
    assert_false(var.can_contain('\n'))
    assert_true(var.must_contain('/'))
    assert_false(var.must_contain('a'))
    assert_true(var.must_startswith('/'))
    assert_false(var.must_startswith('a'))


@with_setup(setup, tear_down)
def test_script():
    get_context().add_var('foo', Variable(name='foo', value=Regexp('.*')))
    var = Variable(name='simple', value='/$foo')
    assert_true(var.depends)
    assert_false(var.regexp)

    assert_false(var.can_startswith('/'))
    assert_false(var.can_startswith('a'))
    assert_true(var.can_contain('/'))
    assert_true(var.can_contain('a'))
    assert_false(var.can_contain('\n'))
    assert_true(var.must_contain('/'))
    assert_false(var.must_contain('a'))
    assert_true(var.must_startswith('/'))
    assert_false(var.must_startswith('a'))


@with_setup(setup, tear_down)
def test_regexp_boundary():
    var = Variable(name='simple', value=Regexp('.*'), boundary=Regexp('/[a-z]', strict=True))
    assert_false(var.depends)
    assert_true(var.regexp)

    assert_true(var.can_startswith('/'))
    assert_false(var.can_startswith('a'))
    assert_false(var.can_contain('/'))
    assert_true(var.can_contain('a'))
    assert_false(var.can_contain('0'))
    assert_false(var.can_contain('\n'))
    assert_true(var.must_contain('/'))
    assert_false(var.must_contain('a'))
    assert_true(var.must_startswith('/'))
    assert_false(var.must_startswith('a'))


@with_setup(setup, tear_down)
def test_script_boundary():
    get_context().add_var('foo', Variable(name='foo', value=Regexp('.*'), boundary=Regexp('[a-z]', strict=True)))
    var = Variable(name='simple', value='/$foo', boundary=Regexp('[/a-z0-9]', strict=True))
    assert_true(var.depends)
    assert_false(var.regexp)

    assert_false(var.can_startswith('/'))
    assert_false(var.can_startswith('a'))
    assert_false(var.can_contain('/'))
    assert_true(var.can_contain('a'))
    assert_false(var.can_contain('\n'))
    assert_false(var.can_contain('0'))
    assert_true(var.must_contain('/'))
    assert_false(var.must_contain('a'))
    assert_true(var.must_startswith('/'))
    assert_false(var.must_startswith('a'))



from nose.tools import assert_equals
from gixy.parser.raw_parser import *


def test_directive():
    config = '''
access_log syslog:server=127.0.0.1,tag=nginx_sentry toolsformat;
user http;
internal;
set $foo "bar";
set $foo 'bar';
proxy_pass http://unix:/run/sock.socket;
rewrite ^/([a-zA-Z0-9]+)$ /$1/${arg_v}.pb break;
    server_name some.tld ~^(www\.)?podberi.(?:ru|com|ua)$
    ~^(www\.)?guru.yandex.ru$;
        '''

    expected = [
        ['access_log', 'syslog:server=127.0.0.1,tag=nginx_sentry', 'toolsformat'],
        ['user', 'http'],
        ['internal'],
        ['set', '$foo', 'bar'],
        ['set', '$foo', 'bar'],
        ['proxy_pass', 'http://unix:/run/sock.socket'],
        ['rewrite', '^/([a-zA-Z0-9]+)$', '/$1/${arg_v}.pb', 'break'],
        ['server_name', 'some.tld', '~^(www\.)?podberi.(?:ru|com|ua)$', '~^(www\.)?guru.yandex.ru$']
    ]

    assert_config(config, expected)


def test_block():
    config = '''
http {
}
        '''

    expected = [['http', [], []]]
    assert_config(config, expected)


def test_block_with_child():
    config = '''
http {
    gzip  on;
}
        '''

    expected = [['http', [], [['gzip', 'on']]]]
    assert_config(config, expected)


def test_location_simple():
    config = '''
location / {
}
location = /foo {
}
location ~ ^/bar {
}
location ~* ^/baz$ {
}
location ^~ ^/bazz {
}
# Whitespace may be omitted:((
location ~\.(js|css)$ {
}
        '''

    expected = [['location', ['/'], []],
                ['location', ['=', '/foo'], []],
                ['location', ['~', '^/bar'], []],
                ['location', ['~*', '^/baz$'], []],
                ['location', ['^~', '^/bazz'], []],
                ['Whitespace may be omitted:(('],
                ['location', ['~', '\.(js|css)$'], []]]

    assert_config(config, expected)


def test_quoted_strings():
    config = '''
some_sq '\\'la\\.\\/\\"';
some_dq '\\'la\\.\\/\\"';
        '''

    expected = [['some_sq', '\'la\\.\\/\"'],
                ['some_dq', '\'la\\.\\/\"']]

    assert_config(config, expected)


def test_location_child():
    config = '''
location = /foo {
     proxy_pass http://unix:/run/sock.socket;
}
        '''

    expected = [['location', ['=', '/foo'], [
        ['proxy_pass', 'http://unix:/run/sock.socket']
    ]]]
    assert_config(config, expected)


def test_nested_location():
    config = '''
location ~* ^/foo {
    location = /foo/bar {
        internal;
        proxy_pass http://any.yandex.ru;
    }

    location = /foo/baz {
        proxy_pass upstream;
    }
}
        '''

    expected = [['location', ['~*', '^/foo'], [
        ['location', ['=', '/foo/bar'], [
            ['internal'],
            ['proxy_pass', 'http://any.yandex.ru']
        ]],
        ['location', ['=', '/foo/baz'], [
            ['proxy_pass', 'upstream']
        ]],
    ]]]

    assert_config(config, expected)


def test_hash_block():
    config = '''
geo $geo {
    default        0;

    127.0.0.1      2;
    192.168.1.0/24 1;
    10.1.0.0/16    1;

    ::1            2;
    2001:0db8::/32 1;
}
        '''

    expected = [['geo', ['$geo'], [
        ['default', '0'],
        ['127.0.0.1', '2'],
        ['192.168.1.0/24', '1'],
        ['10.1.0.0/16', '1'],
        ['::1', '2'],
        ['2001:0db8::/32', '1']
    ]]]

    assert_config(config, expected)


def test_hash_block_in_location():
    config = '''
location /iphone/ {
    types {
      text/html  html htm shtml;
      application/json json;
      application/rss+xml  rss;
      text/vnd.sun.j2me.app-descriptor  jad;
    }
}
        '''

    expected = [['location', ['/iphone/'], [
        ['types', [], [
            ['text/html', 'html', 'htm', 'shtml'],
            ['application/json', 'json'],
            ['application/rss+xml', 'rss'],
            ['text/vnd.sun.j2me.app-descriptor', 'jad']
        ]],
    ]]]

    assert_config(config, expected)


def test_named_location():
    config = '''
location @foo {
    proxy_pass http://any.yandex.ru;
}
        '''

    expected = [['location', ['@foo'], [
        ['proxy_pass', 'http://any.yandex.ru']
    ]]]

    assert_config(config, expected)


def test_if():
    config = '''
# http://nginx.org/ru/docs/http/ngx_http_rewrite_module.html#if

if ($http_user_agent ~ MSIE) {
    rewrite ^(.*)$ /msie/$1 break;
}

if ($http_cookie ~* "id=([^;]+)(?:;|$)") {
    set $id $1;
}

if ($request_method = POST) {
    return 405;
}

if ($slow) {
    limit_rate 10k;
}

if ($invalid_referer) {
    return 403;
}

if (!-e "/var/data/$dataset") {
    return 503;
}

if ($https_or_slb = (by_\(sl\)b|https)) {
}

if ($host ~* (lori|rage2)\.yandex\.(ru|ua|com|com\.tr)) {
    set $x_frame_options ALLOW;
}

if ($request_filename ~* ^.*?/(\d+_)([^/]+)$) {
}

if ($http_user_agent ~* "^WordPress.*; verifying pingback") {
}

if ($foo = "BAR") { rewrite ^(.*)$ /bar; }
        '''

    expected = [
        ['http://nginx.org/ru/docs/http/ngx_http_rewrite_module.html#if'],
        ['if', ['$http_user_agent', '~', 'MSIE'], [
            ['rewrite', '^(.*)$', '/msie/$1', 'break']
        ]],
        ['if', ['$http_cookie', '~*', 'id=([^;]+)(?:;|$)'], [
            ['set', '$id', '$1']
        ]],
        ['if', ['$request_method', '=', 'POST'], [
            ['return', '405']
        ]],
        ['if', ['$slow'], [
            ['limit_rate', '10k']
        ]],
        ['if', ['$invalid_referer'], [
            ['return', '403']
        ]],
        ['if', ['!-e', '/var/data/$dataset'], [
            ['return', '503']
        ]],
        ['if', ['$https_or_slb', '=', '(by_\(sl\)b|https)'], [
        ]],
        ['if', ['$host', '~*', '(lori|rage2)\.yandex\.(ru|ua|com|com\.tr)'], [
            ['set', '$x_frame_options', 'ALLOW']
        ]],
        ['if', ['$request_filename', '~*', '^.*?/(\d+_)([^/]+)$'], [
        ]],
        ['if', ['$http_user_agent', '~*', '^WordPress.*; verifying pingback'], [
        ]],
        ['if', ['$foo', '=', 'BAR'], [
            ['rewrite', '^(.*)$', '/bar']
        ]]
    ]

    assert_config(config, expected)


def test_hash_block_map():
    config = '''
# http://nginx.org/ru/docs/http/ngx_http_map_module.html

map $http_host $name {
    hostnames;

    default       0;

    example.com   1;
    *.example.com 1;
    example.org   2;
    *.example.org 2;
    .example.net  3;
    wap.*         4;
}

map $http_user_agent $mobile {
    default       0;
    "~Opera Mini" 1;
}
        '''

    expected = [
        ['http://nginx.org/ru/docs/http/ngx_http_map_module.html'],
        ['map', ['$http_host', '$name'], [
            ['hostnames'],
            ['default', '0'],
            ['example.com', '1'],
            ['*.example.com', '1'],
            ['example.org', '2'],
            ['*.example.org', '2'],
            ['.example.net', '3'],
            ['wap.*', '4'],
        ]],
        ['map', ['$http_user_agent', '$mobile'], [
            ['default', '0'],
            ['~Opera Mini', '1'],
        ]]
    ]

    assert_config(config, expected)


def test_upstream():
    config = '''
# http://nginx.org/ru/docs/http/ngx_http_upstream_module.html

upstream backend {
    server backend1.example.com       weight=5;
    server backend2.example.com:8080;
    server unix:/tmp/backend3;

    server backup1.example.com:8080   backup;
    server backup2.example.com:8080   backup;
}

server {
    location / {
        proxy_pass http://backend;
    }
}
        '''

    expected = [
        ['http://nginx.org/ru/docs/http/ngx_http_upstream_module.html'],
        ['upstream', ['backend'], [
            ['server', 'backend1.example.com', 'weight=5'],
            ['server', 'backend2.example.com:8080'],
            ['server', 'unix:/tmp/backend3'],
            ['server', 'backup1.example.com:8080', 'backup'],
            ['server', 'backup2.example.com:8080', 'backup'],
        ]],
        ['server', [], [
            ['location', ['/'], [
                ['proxy_pass', 'http://backend']
            ]]
        ]]]

    assert_config(config, expected)


def test_issue_8():
    config = '''
# http://nginx.org/ru/docs/http/ngx_http_upstream_module.html
if ($http_referer ~* (\.(ru|ua|by|kz)/(pages/music|partners/|page-no-rights\.xml)) ) {
    set $temp A;
}
        '''

    expected = [
        ['http://nginx.org/ru/docs/http/ngx_http_upstream_module.html'],
        ['if', ['$http_referer', '~*', '(\.(ru|ua|by|kz)/(pages/music|partners/|page-no-rights\.xml))'], [
            ['set', '$temp', 'A']
        ]]
    ]

    assert_config(config, expected)


def test_issue_11():
    config = '''
init_by_lua_block {
    tvm = require "nginx.tvm"
}
        '''

    expected = [
        ['init_by_lua_block', [], ['tvm', '=', 'require', '"nginx.tvm"']]
    ]

    assert_config(config, expected)


def test_lua_block():
    config = '''
# https://github.com/openresty/lua-nginx-module#typical-uses
location = /lua {
 # MIME type determined by default_type:
 default_type 'text/plain';

 content_by_lua_block {
     local res = ngx.location.capture("/some_other_location")
     if res then
         ngx.say("status: ", res.status)
         ngx.say("body:")
         ngx.print(res.body)
     end
 }
}
        '''

    expected = [
        ['https://github.com/openresty/lua-nginx-module#typical-uses'],
        ['location', ['=', '/lua'], [
            ['MIME type determined by default_type:'],
            ['default_type', 'text/plain'],
            ['content_by_lua_block', [], [
                'local', 'res', '=', 'ngx.location.capture(', '"/some_other_location"', ')',
                'if', 'res', 'then',
                    'ngx.say(', '"status: "', ',', 'res.status)',
                    'ngx.say(', '"body:"', ')',
                    'ngx.print(res.body)',
                'end']]
        ]]
    ]

    assert_config(config, expected)


def test_lua_block_brackets():
    config = '''
location = /foo {
 rewrite_by_lua_block {
     res = ngx.location.capture("/memc",
         { args = { cmd = "incr", key = ngx.var.uri } }
     )
 }

 proxy_pass http://blah.blah.com;
}
        '''

    expected = [
        ['location', ['=', '/foo'], [
            ['rewrite_by_lua_block', [], [
                'res', '=', 'ngx.location.capture(', '"/memc"', ',',
                    ['args', '=', ['cmd', '=', '"incr"', ',', 'key', '=', 'ngx.var.uri']],
                ')']],
            ['proxy_pass', 'http://blah.blah.com']
        ]]
    ]

    assert_config(config, expected)


def test_file_delims():
    config = '''
# configuration file /etc/nginx/nginx.conf:
http {
    include sites/*.conf;
}

# configuration file /etc/nginx/sites/default.conf:
server {

}
        '''

    expected = [
        ['/etc/nginx/nginx.conf'],
        ['http', [], [
            ['include', 'sites/*.conf']
        ]],
        ['/etc/nginx/sites/default.conf'],
        ['server', [], []]
    ]

    assert_config(config, expected)


def test_comments():
    config = '''
# Some comment
add_header X-Some-Comment some;

# 
# Comment with padding
# 
add_header X-Padding-Comment padding;

#
add_header X-Blank-Comment blank;

if (1) # Comment 
{
    add_header X-Inline blank;
}
        '''

    expected = [
        ['Some comment'],
        ['add_header', 'X-Some-Comment', 'some'],
        [''],
        ['Comment with padding'],
        [''],
        ['add_header', 'X-Padding-Comment', 'padding'],
        [''],
        ['add_header', 'X-Blank-Comment', 'blank'],
        ['if', ['1'], [
            ['add_header', 'X-Inline', 'blank'],
        ]],
    ]

    assert_config(config, expected)


def test_upstream_dot():
    config = '''
upstream test.mysite.com {
    server 127.0.0.1:9009;
}
        '''

    expected = [
        ['upstream', ['test.mysite.com'], [
            ['server', '127.0.0.1:9009']
        ]],
    ]

    assert_config(config, expected)


def test_empty_config():
    config = '''
        '''

    expected = []

    assert_config(config, expected)


def test_utfbom_decoding():
    config = b'''\xef\xbb\xbf
add_header X-Test "Windows-1251";
        '''

    expected = [
        ['add_header', 'X-Test', 'Windows-1251']
    ]

    assert_config(config, expected)


def test_national_comment_decoding():
    config = b'''
# \xeb\xff-\xeb\xff-\xeb\xff = Lya-lya-lya
add_header X-Test "Windows-1251";
        '''

    actual = RawParser().parse(config)
    assert_equals(len(actual.asList()), 2)


def assert_config(config, expected):
    actual = RawParser().parse(config)
    assert_equals(actual.asList(), expected)



from nose.tools import assert_equals, assert_false, assert_true
from tests.asserts import assert_is_instance
from gixy.parser.nginx_parser import NginxParser
from gixy.directives.directive import *


def _get_parsed(config):
    root = NginxParser(cwd='', allow_includes=False).parse(config)
    return root.children[0]


def test_directive():
    config = 'some "foo" "bar";'

    directive = _get_parsed(config)
    assert_is_instance(directive, Directive)
    assert_equals(directive.name, 'some')
    assert_equals(directive.args, ['foo', 'bar'])
    assert_equals(str(directive), 'some foo bar;')


def test_add_header():
    config = 'add_header "X-Foo" "bar";'

    directive = _get_parsed(config)
    assert_is_instance(directive, AddHeaderDirective)
    assert_equals(directive.name, 'add_header')
    assert_equals(directive.args, ['X-Foo', 'bar'])
    assert_equals(directive.header, 'x-foo')
    assert_equals(directive.value, 'bar')
    assert_false(directive.always)
    assert_equals(str(directive), 'add_header X-Foo bar;')


def test_add_header_always():
    config = 'add_header "X-Foo" "bar" always;'

    directive = _get_parsed(config)
    assert_is_instance(directive, AddHeaderDirective)
    assert_equals(directive.name, 'add_header')
    assert_equals(directive.args, ['X-Foo', 'bar', 'always'])
    assert_equals(directive.header, 'x-foo')
    assert_equals(directive.value, 'bar')
    assert_true(directive.always)
    assert_equals(str(directive), 'add_header X-Foo bar always;')


def test_set():
    config = 'set $foo bar;'

    directive = _get_parsed(config)
    assert_is_instance(directive, SetDirective)
    assert_equals(directive.name, 'set')
    assert_equals(directive.args, ['$foo', 'bar'])
    assert_equals(directive.variable, 'foo')
    assert_equals(directive.value, 'bar')
    assert_equals(str(directive), 'set $foo bar;')
    assert_true(directive.provide_variables)


def test_rewrite():
    config = 'rewrite ^ http://some;'

    directive = _get_parsed(config)
    assert_is_instance(directive, RewriteDirective)
    assert_equals(directive.name, 'rewrite')
    assert_equals(directive.args, ['^', 'http://some'])
    assert_equals(str(directive), 'rewrite ^ http://some;')
    assert_true(directive.provide_variables)

    assert_equals(directive.pattern, '^')
    assert_equals(directive.replace, 'http://some')
    assert_equals(directive.flag, None)


def test_rewrite_flags():
    config = 'rewrite ^/(.*)$ http://some/$1 redirect;'

    directive = _get_parsed(config)
    assert_is_instance(directive, RewriteDirective)
    assert_equals(directive.name, 'rewrite')
    assert_equals(directive.args, ['^/(.*)$', 'http://some/$1', 'redirect'])
    assert_equals(str(directive), 'rewrite ^/(.*)$ http://some/$1 redirect;')
    assert_true(directive.provide_variables)

    assert_equals(directive.pattern, '^/(.*)$')
    assert_equals(directive.replace, 'http://some/$1')
    assert_equals(directive.flag, 'redirect')


def test_root():
    config = 'root /var/www/html;'

    directive = _get_parsed(config)
    assert_is_instance(directive, RootDirective)
    assert_equals(directive.name, 'root')
    assert_equals(directive.args, ['/var/www/html'])
    assert_equals(str(directive), 'root /var/www/html;')
    assert_true(directive.provide_variables)

    assert_equals(directive.path, '/var/www/html')



from nose.tools import assert_equals, assert_true, assert_false
from tests.asserts import assert_is_instance, assert_is_none, assert_is_not_none
from gixy.parser.nginx_parser import NginxParser
from gixy.directives.block import *

# TODO(buglloc): what about include block?


def _get_parsed(config):
    root = NginxParser(cwd='', allow_includes=False).parse(config)
    return root.children[0]


def test_block():
    config = 'some {some;}'

    directive = _get_parsed(config)
    assert_is_instance(directive, Block)
    assert_true(directive.is_block)
    assert_true(directive.self_context)
    assert_false(directive.provide_variables)


def test_http():
    config = '''
http {
    default_type  application/octet-stream;
    sendfile        on;
    keepalive_timeout  65;
}
    '''

    directive = _get_parsed(config)
    assert_is_instance(directive, HttpBlock)
    assert_true(directive.is_block)
    assert_true(directive.self_context)
    assert_false(directive.provide_variables)


def test_server():
    config = '''
server {
    listen 80;
    server_name _;
    server_name cool.io;
}

    '''

    directive = _get_parsed(config)
    assert_is_instance(directive, ServerBlock)
    assert_true(directive.is_block)
    assert_true(directive.self_context)
    assert_equals([d.args[0] for d in directive.get_names()], ['_', 'cool.io'])
    assert_false(directive.provide_variables)


def test_location():
    config = '''
location / {
}
    '''

    directive = _get_parsed(config)
    assert_is_instance(directive, LocationBlock)
    assert_true(directive.is_block)
    assert_true(directive.self_context)
    assert_true(directive.provide_variables)
    assert_is_none(directive.modifier)
    assert_equals(directive.path, '/')
    assert_false(directive.is_internal)


def test_location_internal():
    config = '''
location / {
    internal;
}
    '''

    directive = _get_parsed(config)
    assert_is_instance(directive, LocationBlock)
    assert_true(directive.is_internal)


def test_location_modifier():
    config = '''
location = / {
}
    '''

    directive = _get_parsed(config)
    assert_is_instance(directive, LocationBlock)
    assert_equals(directive.modifier, '=')
    assert_equals(directive.path, '/')


def test_if():
    config = '''
if ($some) {
}
    '''

    directive = _get_parsed(config)
    assert_is_instance(directive, IfBlock)
    assert_true(directive.is_block)
    assert_false(directive.self_context)
    assert_false(directive.provide_variables)
    assert_equals(directive.variable, '$some')
    assert_is_none(directive.operand)
    assert_is_none(directive.value)


def test_if_modifier():
    config = '''
if (-f /some) {
}
    '''

    directive = _get_parsed(config)
    assert_is_instance(directive, IfBlock)
    assert_equals(directive.operand, '-f')
    assert_equals(directive.value, '/some')
    assert_is_none(directive.variable)


def test_if_variable():
    config = '''
if ($http_some = '/some') {
}
    '''

    directive = _get_parsed(config)
    assert_is_instance(directive, IfBlock)
    assert_equals(directive.variable, '$http_some')
    assert_equals(directive.operand, '=')
    assert_equals(directive.value, '/some')


def test_block_some_flat():
    config = '''
    some {
        default_type  application/octet-stream;
        sendfile        on;
        if (-f /some/) {
            keepalive_timeout  65;
        }
    }
        '''

    directive = _get_parsed(config)
    for d in ['default_type', 'sendfile', 'keepalive_timeout']:
        c = directive.some(d, flat=True)
        assert_is_not_none(c)
        assert_equals(c.name, d)


def test_block_some_not_flat():
    config = '''
    some {
        default_type  application/octet-stream;
        sendfile        on;
        if (-f /some/) {
            keepalive_timeout  65;
        }
    }
        '''

    directive = _get_parsed(config)
    c = directive.some('keepalive_timeout', flat=False)
    assert_is_none(c)


def test_block_find_flat():
    config = '''
    some {
        directive 1;
        if (-f /some/) {
            directive 2;
        }
    }
        '''

    directive = _get_parsed(config)
    finds = directive.find('directive', flat=True)
    assert_equals(len(finds), 2)
    assert_equals([x.name for x in finds], ['directive', 'directive'])
    assert_equals([x.args[0] for x in finds], ['1', '2'])


def test_block_find_not_flat():
    config = '''
    some {
        directive 1;
        if (-f /some/) {
            directive 2;
        }
    }
        '''

    directive = _get_parsed(config)
    finds = directive.find('directive', flat=False)
    assert_equals(len(finds), 1)
    assert_equals([x.name for x in finds], ['directive'])
    assert_equals([x.args[0] for x in finds], ['1'])


def test_block_map():
    config = '''
map $some_var $some_other_var {
    a   b;
    default c;
}
    '''

    directive = _get_parsed(config)
    assert_is_instance(directive, MapBlock)
    assert_true(directive.is_block)
    assert_false(directive.self_context)
    assert_true(directive.provide_variables)
    assert_equals(directive.variable, 'some_other_var')


def test_block_geo_two_vars():
    config = '''
geo $some_var $some_other_var {
    1.2.3.4 b;
    default c;
}
    '''

    directive = _get_parsed(config)
    assert_is_instance(directive, GeoBlock)
    assert_true(directive.is_block)
    assert_false(directive.self_context)
    assert_true(directive.provide_variables)
    assert_equals(directive.variable, 'some_other_var')


def test_block_geo_one_var():
    config = '''
geo $some_var {
    5.6.7.8 d;
    default e;
}
    '''

    directive = _get_parsed(config)
    assert_is_instance(directive, GeoBlock)
    assert_true(directive.is_block)
    assert_false(directive.self_context)
    assert_true(directive.provide_variables)
    assert_equals(directive.variable, 'some_var')



from nose.tools import with_setup, assert_equals, assert_not_equals, assert_true
from gixy.core.context import get_context, pop_context, push_context, purge_context, CONTEXTS, Context
from gixy.directives.block import Root
from gixy.core.variable import Variable
from gixy.core.regexp import Regexp


def setup():
    assert_equals(len(CONTEXTS), 0)


def tear_down():
    purge_context()


@with_setup(setup, tear_down)
def test_push_pop_context():
    root_a = Root()
    push_context(root_a)
    assert_equals(len(CONTEXTS), 1)
    root_b = Root()
    push_context(root_b)
    assert_equals(len(CONTEXTS), 2)

    poped = pop_context()
    assert_equals(len(CONTEXTS), 1)
    assert_equals(poped.block, root_b)
    poped = pop_context()
    assert_equals(len(CONTEXTS), 0)
    assert_equals(poped.block, root_a)


@with_setup(setup, tear_down)
def test_push_get_purge_context():
    root = Root()
    push_context(root)
    assert_equals(len(CONTEXTS), 1)
    assert_equals(get_context().block, root)
    root = Root()
    push_context(root)
    assert_equals(len(CONTEXTS), 2)
    assert_equals(get_context().block, root)

    purge_context()
    assert_equals(len(CONTEXTS), 0)


@with_setup(setup, tear_down)
def test_add_variables():
    context = push_context(Root())
    assert_equals(len(context.variables['index']), 0)
    assert_equals(len(context.variables['name']), 0)

    one_str_var = Variable('1')
    context.add_var('1', one_str_var)
    one_int_var = Variable(1)
    context.add_var(1, one_int_var)
    some_var = Variable('some')
    context.add_var('some', some_var)

    assert_equals(len(context.variables['index']), 1)
    assert_equals(context.variables['index'][1], one_int_var)
    assert_equals(len(context.variables['name']), 1)
    assert_equals(context.variables['name']['some'], some_var)
    context.clear_index_vars()
    assert_equals(len(context.variables['index']), 0)
    assert_equals(len(context.variables['name']), 1)
    assert_equals(context.variables['name']['some'], some_var)


@with_setup(setup, tear_down)
def test_get_variables():
    context = push_context(Root())
    assert_equals(len(context.variables['index']), 0)
    assert_equals(len(context.variables['name']), 0)

    one_var = Variable(1)
    context.add_var(1, one_var)
    some_var = Variable('some')
    context.add_var('some', some_var)

    assert_equals(context.get_var(1), one_var)
    assert_equals(context.get_var('some'), some_var)
    # Checks not existed variables, for now context may return None
    assert_equals(context.get_var(0), None)
    assert_equals(context.get_var('not_existed'), None)
    # Checks builtins variables
    assert_true(context.get_var('uri'))
    assert_true(context.get_var('document_uri'))
    assert_true(context.get_var('arg_asdsadasd'))
    assert_true(context.get_var('args'))


@with_setup(setup, tear_down)
def test_context_depend_variables():
    push_context(Root())
    assert_equals(len(get_context().variables['index']), 0)
    assert_equals(len(get_context().variables['name']), 0)

    get_context().add_var(1, Variable(1, value='one'))
    get_context().add_var('some', Variable('some', value='some'))

    assert_equals(get_context().get_var(1).value, 'one')
    assert_equals(get_context().get_var('some').value, 'some')

    # Checks top context variables are still exists
    push_context(Root())
    assert_equals(get_context().get_var(1).value, 'one')
    assert_equals(get_context().get_var('some').value, 'some')

    # Checks variable overriding
    get_context().add_var('some', Variable('some', value='some_new'))
    get_context().add_var('foo', Variable('foo', value='foo'))
    assert_not_equals(get_context().get_var('some').value, 'some')
    assert_equals(get_context().get_var('some').value, 'some_new')
    assert_equals(get_context().get_var('foo').value, 'foo')
    assert_equals(get_context().get_var(1).value, 'one')

    # Checks variables after restore previous context
    pop_context()
    assert_not_equals(get_context().get_var('some').value, 'some_new')
    assert_equals(get_context().get_var('some').value, 'some')
    assert_equals(get_context().get_var('foo'), None)
    assert_equals(get_context().get_var(1).value, 'one')


@with_setup(setup, tear_down)
def test_push_failed_with_regexp_py35_gixy_10():
    push_context(Root())
    assert_equals(len(get_context().variables['index']), 0)
    assert_equals(len(get_context().variables['name']), 0)

    regexp = Regexp('^/some/(.*?)')
    for name, group in regexp.groups.items():
        get_context().add_var(name, Variable(name=name, value=group))

    push_context(Root())



from nose.tools import assert_equals, assert_true
from tests.asserts import assert_in
import os
from os import path
import json

from ..utils import *
from gixy.core.manager import Manager as Gixy
from gixy.core.plugins_manager import PluginsManager
from gixy.core.config import Config


def setup_module():
    pass


def teardown_module():
    pass


def test_from_config():
    tested_plugins = set()
    tested_fp_plugins = set()

    conf_dir = path.join(path.dirname(__file__), 'simply')
    for plugin in os.listdir(conf_dir):
        if plugin in ('.', '..'):
            continue

        plugin_path = path.join(conf_dir, plugin)
        if not path.isdir(plugin_path):
            continue

        config = {}
        if path.exists(path.join(plugin_path, 'config.json')):
            with open(path.join(plugin_path, 'config.json'), 'r') as file:
                config = json.loads(file.read())

        for test_case in os.listdir(plugin_path):
            if not test_case.endswith('.conf'):
                continue

            config_path = path.join(plugin_path, test_case)
            if not test_case.endswith('_fp.conf'):
                # Not False Positive test
                tested_plugins.add(plugin)
                test_func = check_configuration
            else:
                tested_fp_plugins.add(plugin)
                test_func = check_configuration_fp

            yield test_func, plugin, config_path, config

    manager = PluginsManager()
    for plugin in manager.plugins:
        plugin = plugin.name
        assert_true(plugin in tested_plugins,
                    'Plugin {name!r} should have at least one simple test config'.format(name=plugin))
        assert_true(plugin in tested_fp_plugins,
                    'Plugin {name!r} should have at least one simple test config with false positive'.format(name=plugin))


def parse_plugin_options(config_path):
    with open(config_path, 'r') as f:
        config_line = f.readline()
        if config_line.startswith('# Options: '):
            return json.loads(config_line[10:])
    return None


def yoda_provider(plugin, plugin_options=None):
    config = Config(
        allow_includes=False,
        plugins=[plugin]
    )
    if plugin_options:
        config.set_for(plugin, plugin_options)
    return Gixy(config=config)


def check_configuration(plugin, config_path, test_config):
    plugin_options = parse_plugin_options(config_path)
    with yoda_provider(plugin, plugin_options) as yoda:
        yoda.audit(config_path, open(config_path, mode='r'))
        formatter = BaseFormatter()
        formatter.feed(config_path, yoda)
        _, results = formatter.reports.popitem()

        assert_equals(len(results), 1, 'Should have one report')
        result = results[0]

        if 'severity' in test_config:
            if not hasattr(test_config['severity'], '__iter__'):
                assert_equals(result['severity'], test_config['severity'])
            else:
                assert_in(result['severity'], test_config['severity'])
        assert_equals(result['plugin'], plugin)
        assert_true(result['summary'])
        assert_true(result['description'])
        assert_true(result['config'])
        assert_true(result['help_url'].startswith('https://'),
                    'help_url must starts with https://. It\'is URL!')


def check_configuration_fp(plugin, config_path, test_config):
    with yoda_provider(plugin) as yoda:
        yoda.audit(config_path, open(config_path, mode='r'))
        assert_equals(len([x for x in yoda.results]), 0,
                      'False positive configuration must not trigger any plugins')



from __future__ import absolute_import
from six import PY3, text_type, binary_type


def to_bytes(obj, encoding='latin1', errors='strict', nonstring='replace'):
    if isinstance(obj, binary_type):
        return obj

    if isinstance(obj, text_type):
        try:
            # Try this first as it's the fastest
            return obj.encode(encoding, errors)
        except UnicodeEncodeError:
            return b'failed_to_encode'

    if nonstring == 'simplerepr':
        try:

            value = str(obj)
        except UnicodeError:
            try:
                value = repr(obj)
            except UnicodeError:
                # Giving up
                return b'failed_to_encode'
    elif nonstring == 'passthru':
        return obj
    elif nonstring == 'replace':
        return b'failed_to_encode'
    elif nonstring == 'strict':
        raise TypeError('obj must be a string type')
    else:
        raise TypeError('Invalid value %s for to_bytes\' nonstring parameter' % nonstring)

    return to_bytes(value, encoding, errors)


def to_text(obj, encoding='latin1', errors='strict', nonstring='replace'):
    if isinstance(obj, text_type):
        return obj

    if isinstance(obj, binary_type):
        try:
            return obj.decode(encoding, errors)
        except UnicodeEncodeError:
            return u'failed_to_encode'

    if nonstring == 'simplerepr':
        try:
            value = str(obj)
        except UnicodeError:
            try:
                value = repr(obj)
            except UnicodeError:
                # Giving up
                return u'failed_to_encode'
    elif nonstring == 'passthru':
        return obj
    elif nonstring == 'replace':
        return u'failed_to_encode'
    elif nonstring == 'strict':
        raise TypeError('obj must be a string type')
    else:
        raise TypeError('Invalid value %s for to_text\'s nonstring parameter' % nonstring)

    return to_text(value, encoding, errors)


if PY3:
    to_native = to_text
else:
    to_native = to_bytes



from __future__ import absolute_import

import json

from gixy.formatters.base import BaseFormatter


class JsonFormatter(BaseFormatter):
    def format_reports(self, reports, stats):
        result = []
        for path, issues in reports.items():
            for issue in issues:
                result.append(dict(
                    path=path,
                    plugin=issue['plugin'],
                    summary=issue['summary'],
                    severity=issue['severity'],
                    description=issue['description'],
                    reference=issue['help_url'],
                    reason=issue['reason'],
                    config=issue['config']
                ))

        return json.dumps(result, sort_keys=True, indent=2, separators=(',', ': '))



from __future__ import absolute_import
from jinja2 import Environment, PackageLoader

from gixy.utils.text import to_text


def load_template(name):
    env = Environment(loader=PackageLoader('gixy', 'formatters/templates'), trim_blocks=True, lstrip_blocks=True)
    env.filters['to_text'] = to_text_filter
    return env.get_template(name)


def to_text_filter(text):
    try:
        return text.encode('latin1').decode('utf-8')
    except UnicodeEncodeError:
        return to_text(text)



from nose.tools import assert_true, assert_false


'''
Various nose.tools helpers that doesn't exists in Python 2.6 Unittest :(
Must be removed with drop Python 2.6 support
'''


def assert_is_instance(obj, cls, msg=None):
    """Same as assert_true(isinstance(obj, cls)), with a nicer
    default message."""
    if not msg:
        msg = '{orig} is not an instance of {test}'.format(orig=type(obj), test=cls)
    assert_true(isinstance(obj, cls), msg=msg)


def assert_is_none(obj, msg=None):
    """Same as assert_true(obj is None), with a nicer default message."""
    if not msg:
        msg = '{orig!r} is not None'.format(orig=obj)
    assert_true(obj is None, msg=msg)


def assert_is_not_none(obj, msg=None):
    """Same as assert_false(obj is None), with a nicer default message."""
    if not msg:
        msg = '{orig!r} is None'.format(orig=obj)
    assert_false(obj is None, msg=msg)


def assert_in(member, container, msg=None):
    """Just like assert_true(a in b), but with a nicer default message."""
    if not msg:
        msg = '{member!r} not found in {container!r}'.format(member=member, container=container)
    assert_true(member in container, msg=msg)



from nose.tools import assert_equal
from tests.asserts import assert_is_instance
from gixy.parser.nginx_parser import NginxParser
from gixy.directives.directive import *
from gixy.directives.block import *


def _parse(config):
    return NginxParser(cwd='', allow_includes=False).parse(config)


def test_directive():
    configs = [
        'access_log syslog:server=127.0.0.1,tag=nginx_sentry toolsformat;',
        'user http;',
        'internal;',
        'set $foo "bar";',
        "set $foo 'bar';",
        'proxy_pass http://unix:/run/sock.socket;',
        'rewrite ^/([a-zA-Z0-9]+)$ /$1/${arg_v}.pb break;'
    ]

    expected = [
        [Directive],
        [Directive],
        [Directive],
        [Directive, SetDirective],
        [Directive],
        [Directive, RewriteDirective]
    ]

    for i, config in enumerate(configs):
        return assert_config, config, expected[i]


def test_blocks():
    configs = [
        'if (-f /some) {}',
        'location / {}'
    ]

    expected = [
        [Directive, Block, IfBlock],
        [Directive, Block, LocationBlock],
    ]

    for i, config in enumerate(configs):
        yield assert_config, config, expected[i]


def test_dump_simple():
    config = '''
# configuration file /etc/nginx/nginx.conf:
http {
    include sites/*.conf;
}

# configuration file /etc/nginx/conf.d/listen:
listen 80;

# configuration file /etc/nginx/sites/default.conf:
server {
    include conf.d/listen;
}
    '''

    tree = _parse(config)
    assert_is_instance(tree, Directive)
    assert_is_instance(tree, Block)
    assert_is_instance(tree, Root)

    assert_equal(len(tree.children), 1)
    http = tree.children[0]
    assert_is_instance(http, Directive)
    assert_is_instance(http, Block)
    assert_is_instance(http, HttpBlock)

    assert_equal(len(http.children), 1)
    include_server = http.children[0]
    assert_is_instance(include_server, Directive)
    assert_is_instance(include_server, IncludeBlock)
    assert_equal(include_server.file_path, '/etc/nginx/sites/default.conf')

    assert_equal(len(include_server.children), 1)
    server = include_server.children[0]
    assert_is_instance(server, Directive)
    assert_is_instance(server, Block)
    assert_is_instance(server, ServerBlock)

    assert_equal(len(server.children), 1)
    include_listen = server.children[0]
    assert_is_instance(include_listen, Directive)
    assert_is_instance(include_listen, IncludeBlock)
    assert_equal(include_listen.file_path, '/etc/nginx/conf.d/listen')

    assert_equal(len(include_listen.children), 1)
    listen = include_listen.children[0]
    assert_is_instance(listen, Directive)
    assert_equal(listen.args, ['80'])


def test_encoding():
    configs = [
        'bar "\xD1\x82\xD0\xB5\xD1\x81\xD1\x82";'
    ]

    for i, config in enumerate(configs):
        _parse(config)


def assert_config(config, expected):
    tree = _parse(config)
    assert_is_instance(tree, Directive)
    assert_is_instance(tree, Block)
    assert_is_instance(tree, Root)

    child = tree.children[0]
    for ex in expected:
        assert_is_instance(child, ex)



import gixy
from gixy.plugins.plugin import Plugin


class alias_traversal(Plugin):
    """
    Insecure example:
        location /files {
            alias /home/;
        }
    """
    summary = 'Path traversal via misconfigured alias.'
    severity = gixy.severity.HIGH
    description = 'Using alias in a prefixed location that doesn\'t ends with directory separator could lead to path ' \
                  'traversal vulnerability. '
    help_url = 'https://github.com/yandex/gixy/blob/master/docs/en/plugins/aliastraversal.md'
    directives = ['alias']

    def audit(self, directive):
        for location in directive.parents:
            if location.name != 'location':
                continue

            if not location.modifier or location.modifier == '^~':
                # We need non-strict prefixed locations
                if not location.path.endswith('/'):
                    self.add_issue(
                        severity=gixy.severity.HIGH if directive.path.endswith('/') else gixy.severity.MEDIUM,
                        directive=[directive, location]
                    )
            break
